#include <QMainWindow>
#include <QDockWidget>
#include <QtXml>
#include "notepad_editor.h"
#include "tab_docs.h"
#include "savedialog.h"

notepad_editor_class::notepad_editor_class(QWidget *parent)
    : QMainWindow(parent)
{
    _tabWid = new QTabWidget(this);
    _tabWid->setIconSize(QSize(12,12));
    setCentralWidget(_tabWid);
    _tabWid->setTabsClosable(true);
    connect(_tabWid, SIGNAL(tabCloseRequested(int)), this, SLOT(sltCloseIdx(int)));

    _newTabsCount = 0;
    _amountOfChanged = 0;
    QAction* actSave = new QAction(QIcon(":pics/Data/Save.png"), "Сохранить", this);
    actSave->setShortcut(QKeySequence("CTRL+S"));
    QAction* actSaveAll = new QAction(QIcon(":pics/Data/SaveAll.png"), "Сохранить все", this);
    actSaveAll->setShortcut(QKeySequence("CTRL+SHIFT+S"));
    QAction* actClose = new QAction(QIcon(":pics/Data/Close.png"), "Закрыть", this);
    actClose->setShortcut(QKeySequence("CTRL+D"));
    QAction* actNew = new QAction(QIcon(":pics/Data/New.png"), "Новый...", this);
    actNew->setShortcut(QKeySequence("CTRL+N"));
    QAction* actOpen = new QAction(QIcon(":pics/Data/Open.png"), "Открыть...", this);
    actOpen->setShortcut(QKeySequence("CTRL+O"));

    QMenu* fileMenu = new QMenu(tr("Файл"), this);
    QMenu* editMenu = new QMenu(tr("Редактировать"), this);
    QMenu* viewMenu = new QMenu(tr("Вид"), this);

    connect(actNew, &QAction::triggered, this, &notepad_editor_class::sltNewDocument);
    connect(actOpen, &QAction::triggered, this, &notepad_editor_class::sltOpen);
    connect(actSave, &QAction::triggered, this, &notepad_editor_class::sltSave);
    connect(actSaveAll, &QAction::triggered, this, &notepad_editor_class::sltSaveAll);
    connect(actClose, SIGNAL(triggered()), this, SLOT(sltClose()));


    fileMenu->addAction(actNew);
    fileMenu->addAction(actOpen);
    fileMenu->addAction(actSave);
    fileMenu->addAction(actSaveAll);
    fileMenu->addAction(actClose);
    fileMenu->addAction(QIcon(":pics/Data/SaveAs.png"), tr("Сохранить как..."), this, &notepad_editor_class::sltSaveAs);
    fileMenu->addAction(QIcon(":pics/Data/Close.png"), tr("Закрыть все"), this, &notepad_editor_class::sltCloseAll, QKeySequence("CTRL+SHIFT+D"));
    fileMenu->addAction(QIcon(":pics/Data/Exit.png"), tr("Выход"), this, &notepad_editor_class::sltOnClose, QKeySequence("CTRL+Q"));

    viewMenu->addAction(tr("Локальный диск"), this, &notepad_editor_class::sltFileBrowser, QKeySequence("CTRL+SHIFT+B"));
    viewMenu->addAction(tr("Отображать вкладки"), this, &notepad_editor_class::sltTabs, QKeySequence("CTRL+SHIFT+T"));

    editMenu->addAction("Копировать", this, &notepad_editor_class::sltCopy);
    editMenu->addAction("Вырезать", this, &notepad_editor_class::sltCut);
    editMenu->addAction("Вставить", this, &notepad_editor_class::sltPaste);
    editMenu->addAction("Удалить", this, &notepad_editor_class::sltDelete);
    editMenu->addAction("Выбрать все", this, &notepad_editor_class::sltSelectAll);

    menuBar()->addMenu(fileMenu);
    menuBar()->addMenu(editMenu);
    menuBar()->addMenu(viewMenu);

    QToolBar* toolBarFile = new QToolBar(this);
    toolBarFile->addAction(actNew);
    toolBarFile->addAction(actOpen);
    toolBarFile->addAction(actSave);
    toolBarFile->addAction(actSaveAll);
    toolBarFile->addAction(actClose);
    toolBarFile->setIconSize(QSize(35, 35));
    addToolBar(Qt::TopToolBarArea, toolBarFile);
    resize(1500, 1200);

    _dockTabsWid = new QDockWidget("Вкладки", this);
    _dockTabsWid->hide();
    _tableTabsView = new QListView(_dockTabsWid);
    _tableTabsView->setEditTriggers(QTableView::NoEditTriggers);
    _tableTabsView->setSelectionMode(QAbstractItemView::SingleSelection);
    _strListModel = new QStringListModel(_tableTabsView);
    this->addDockWidget(Qt::DockWidgetArea::LeftDockWidgetArea, _dockTabsWid);
    connect(_tableTabsView, SIGNAL(clicked(const QModelIndex&)), this, SLOT(sltSetActiveTab(const QModelIndex&)));
    _dockFileSysWid = new QDockWidget("Диск", this);
    _dockFileSysWid->hide();

    _treeFileSysView = new QTreeView(_dockFileSysWid);
    _sysModel = new QFileSystemModel(_treeFileSysView);
    _sysModel->setRootPath(QDir::rootPath());
    this->addDockWidget(Qt::DockWidgetArea::LeftDockWidgetArea, _dockFileSysWid);
    connect(_treeFileSysView, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(sltOpenIdx(const QModelIndex&)));
    getExtensionsFromIni();
}

notepad_editor_class::~notepad_editor_class()
{}

void notepad_editor_class::getExtensionsFromIni()
{
    QDomDocument domDoc;
    QFile hLIni(":/hLIni/highlightTest.xml");

    if (hLIni.open(QIODevice::ReadOnly))
    {
        if (domDoc.setContent(&hLIni))
        {


            auto domNodeExtList = domDoc.elementsByTagName("syntax");
            qDebug() << domNodeExtList.count();
            for (int at = 0; at < domNodeExtList.count(); at++)
            {
                auto attrMap = domNodeExtList.item(at).attributes();

                for(int atAttribute = 0; atAttribute < attrMap.length(); atAttribute++)
                {
                    if(attrMap.item(atAttribute).toAttr().name() == "ext_list")
                    {
                        QString singleLangExt = attrMap.item(atAttribute).toAttr().value();
                        qDebug() << singleLangExt;
                        auto singleLangExtList = singleLangExt.split(" ");
                        _codeExtensions << singleLangExtList;
                        qDebug() << _codeExtensions;
                    }

                }
            }
        }
    }
    hLIni.close();
    emit  statusBar()->showMessage(tr("Загружена расцветка текста из highlightTest.xml."));
}

void notepad_editor_class::closeEvent(QCloseEvent *event)
{
    if(sltCloseAll())
        event->accept();
    else
        event->ignore();
}

doc_tab_class* notepad_editor_class::createNewDocument()
{
    doc_tab_class* newTab = new doc_tab_class();
    _newTabName = QString("New Document(%1)").arg(++_newTabsCount);
    newTab->setFileName(_newTabName);
    connect(newTab, SIGNAL(changeWidTitle(const QString&, const QString&)), this, SLOT(sltChangeTabTitle(const QString&, const QString&)));
    connect(newTab, SIGNAL(fileChanded(const QString&)), this, SLOT(sltFileChanged(const QString&)));
    newTab->linesHelp();
    return newTab;
}

void notepad_editor_class::sltNewDocument()
{
    doc_tab_class* newTab = createNewDocument();
    _tabWid->addTab(newTab, _newTabName);
    _tabWid->setCurrentWidget(newTab);
    qDebug() << "Added" << _newTabName << "to _openedTabsNames :";
    _openedTabsNames << _newTabName;
    qDebug() << _openedTabsNames;
    _openedTabsDirectories << newTab->getFileDirectory();
    _strListModel->setStringList(_openedTabsNames);

    emit  statusBar()->showMessage(tr("Made a new document named ") + _newTabName + ".");
}

void notepad_editor_class::sltChangeTabTitle(const QString& p_lastName, const QString& p_lastDirectory)
{
    doc_tab_class* senderPtr = qobject_cast<doc_tab_class*>(sender());
    if(!senderPtr)
        return;

    if(p_lastName.isEmpty())
        _tabWid->setTabText(_tabWid->indexOf(senderPtr), senderPtr->getFileName());
    else
    {
        _openedTabsNames.replace(_openedTabsNames.indexOf(p_lastName), senderPtr->getFileName());
        _openedTabsDirectories.replace(_openedTabsDirectories.indexOf(p_lastDirectory), senderPtr->getFileDirectory());
        _tabWid->setTabText(_tabWid->indexOf(senderPtr), senderPtr->getFileName());
    }
}

void notepad_editor_class::sltOpen()
{
    doc_tab_class* newTab = createNewDocument();
    QString newTabName = newTab->sltLoad();

    if(newTabName.isEmpty())
        return;

    if(_openedTabsNames.contains(newTabName))
        return;


    _tabWid->addTab(newTab, newTabName);
    _openedTabsNames << newTabName;

    if(_codeExtensions.contains(newTab->getFileExt()))
        newTab->sltIsCodeDoc();

    qDebug() << newTab->getFileDirectory();
    _openedTabsDirectories << newTab->getFileDirectory();
    _strListModel->setStringList(_openedTabsNames);
    newTab->show();
    _tabWid->setCurrentWidget(newTab);

    qDebug() << _openedTabsDirectories << _openedTabsNames;
    emit  statusBar()->showMessage(tr("Opened the document named ") + newTabName + ".");
}

void notepad_editor_class::sltOpenIdx(const QModelIndex& p_idx)
{
    doc_tab_class* newTab = createNewDocument();
    QString newTabName = newTab->sltLoad(_sysModel->filePath(p_idx));

    if(_openedTabsNames.contains(newTabName))
        return;

    _tabWid->addTab(newTab, newTabName);
    _openedTabsNames << newTabName;

    if(_codeExtensions.contains(newTab->getFileExt()))
        newTab->sltIsCodeDoc();
    qDebug() << newTab->getFileDirectory();
    _openedTabsDirectories << newTab->getFileDirectory();
    _strListModel->setStringList(_openedTabsNames);
    newTab->show();
    _tabWid->setCurrentWidget(newTab);

    qDebug() << _openedTabsDirectories << _openedTabsNames;

    emit  statusBar()->showMessage(tr("Opened the document named ") + newTabName + ".");
}

void notepad_editor_class::sltSave()
{
    doc_tab_class* currentTab = qobject_cast<doc_tab_class*>(_tabWid->currentWidget());
    if(currentTab)
    {
        if(!currentTab->wasChanged())
            return;

        currentTab->sltSave();
        _amountOfChanged--;
        _tabWid->setTabIcon(_tabWid->indexOf(currentTab), QIcon());
        emit  statusBar()->showMessage(tr("Saved the document named ") + currentTab->getFileName() + ".");
    }
}

void notepad_editor_class::sltSaveAs()
{
    doc_tab_class* currentTab = qobject_cast<doc_tab_class*>(_tabWid->currentWidget());
    if(currentTab)
    {
        currentTab->sltSaveAs(currentTab->getFileName());
        _amountOfChanged--;
        _tabWid->setTabIcon(_tabWid->indexOf(currentTab), QIcon());
        emit  statusBar()->showMessage(tr("Saved the document with new name ") + currentTab->getFileName() + ".");
    }

}

void notepad_editor_class::sltSaveAll()
{
    for(int i = 0; i < _tabWid->count(); i++)
    {
        doc_tab_class* currentTab = qobject_cast<doc_tab_class*>(_tabWid->widget(i));
        _tabWid->setTabIcon(_tabWid->indexOf(currentTab), QIcon());
        if (currentTab)
        {
            if(currentTab->wasChanged())
                currentTab->sltSave();
        }
    }
    _amountOfChanged = 0;
    emit  statusBar()->showMessage(tr("Saved all documents."));

}

void notepad_editor_class::sltClose()
{
    int currentIdx = _tabWid->currentIndex();
    QString fileName = _tabWid->tabText(currentIdx);
    doc_tab_class* currentTab = qobject_cast<doc_tab_class*>(_tabWid->currentWidget());

    if(!currentTab)
        return;

    if(!currentTab->wasChanged())
    {
         _tabWid->removeTab(currentIdx);
         _openedTabsNames.removeAll(fileName);
         _openedTabsDirectories.removeAll(currentTab->getFileDirectory());
         _strListModel->setStringList(_openedTabsNames);
         qDebug() << _openedTabsDirectories << _openedTabsNames;
         emit  statusBar()->showMessage(tr("Closed the document with name ") + currentTab->getFileName() + ".");
         delete currentTab;
         return;
    }

    QMessageBox* saveMsg = new QMessageBox(this);
    saveMsg->setText("Save changes?");
    saveMsg->setInformativeText("Сохранить изменения " + currentTab->getFileName() + "?");
    saveMsg->setStandardButtons(QMessageBox::Save | QMessageBox::StandardButton::No | QMessageBox::Cancel);
    saveMsg->setDefaultButton(QMessageBox::Save);
    int option = saveMsg->exec();
    switch (option) {
      case QMessageBox::Save:
          currentTab->sltSave();
          _amountOfChanged--;
          _tabWid->removeTab(currentIdx);
          _openedTabsNames.removeAll(fileName);
          _openedTabsDirectories.removeAll(currentTab->getFileDirectory());
          _strListModel->setStringList(_openedTabsNames);
          emit  statusBar()->showMessage(tr("Closed the document with name ") + currentTab->getFileName() + ".");
          delete currentTab;
          break;

      case QMessageBox::No:
          _amountOfChanged--;
          _tabWid->removeTab(currentIdx);
          _openedTabsNames.removeAll(fileName);
          _openedTabsDirectories.removeAll(currentTab->getFileDirectory());
          _strListModel->setStringList(_openedTabsNames);
          emit  statusBar()->showMessage(tr("Closed the document with name ") + currentTab->getFileName() + ".");
          delete currentTab;
          break;
      case QMessageBox::Cancel:
          break;
      default:
          break;
    }

    qDebug() << _openedTabsDirectories << _openedTabsNames;

}

void notepad_editor_class::sltCloseIdx(int p_tab)
{
    int currentIdx = p_tab;
    QString fileName = _tabWid->tabText(currentIdx);
    doc_tab_class* currentTab = qobject_cast<doc_tab_class*>(_tabWid->widget(currentIdx));

    if(!currentTab->wasChanged())
    {
         _tabWid->removeTab(currentIdx);
         _openedTabsNames.removeAll(fileName);
         _openedTabsDirectories.removeAll(currentTab->getFileDirectory());
         _strListModel->setStringList(_openedTabsNames);
         qDebug() << _openedTabsDirectories << _openedTabsNames;
         emit  statusBar()->showMessage(tr("Closed the document with name ") + currentTab->getFileName() + ".");
         delete currentTab;
         return;
    }

    QMessageBox* saveMsg = new QMessageBox(this);
    saveMsg->setText("Save changes?");
    saveMsg->setInformativeText("Do you want to save changes to file " + currentTab->getFileName() + "?");
    saveMsg->setStandardButtons(QMessageBox::Save | QMessageBox::StandardButton::No | QMessageBox::Cancel);
    saveMsg->setDefaultButton(QMessageBox::Save);
    int option = saveMsg->exec();
    switch (option) {
      case QMessageBox::Save:
          currentTab->sltSave();
          _amountOfChanged--;
          _tabWid->removeTab(currentIdx);
          _openedTabsNames.removeAll(fileName);
          _openedTabsDirectories.removeAll(currentTab->getFileDirectory());
          _strListModel->setStringList(_openedTabsNames);
          emit  statusBar()->showMessage(tr("Closed the document with name ") + currentTab->getFileName() + ".");
          delete currentTab;
          break;
      case QMessageBox::No:
          _amountOfChanged--;
          _tabWid->removeTab(currentIdx);
          _openedTabsNames.removeAll(fileName);
          _openedTabsDirectories.removeAll(currentTab->getFileDirectory());
          _strListModel->setStringList(_openedTabsNames);
          emit  statusBar()->showMessage(tr("Closed the document with name ") + currentTab->getFileName() + ".");
          delete currentTab;
          break;
      case QMessageBox::Cancel:
          break;
      default:
          break;
    }


    qDebug() << _openedTabsDirectories << _openedTabsNames;


}

void notepad_editor_class::sltSetActiveTab(QWidget* p_tab)
{
    _tabWid->setCurrentWidget(p_tab);
}

void notepad_editor_class::sltSetActiveTab(const QModelIndex& p_idx)
{
    QString name = _strListModel->data(p_idx).toString();
    _tabWid->setCurrentIndex(_openedTabsNames.indexOf(name));
}

bool notepad_editor_class::sltCloseAll()
{
    if(_tabWid->count() == 0 || _amountOfChanged == 0)
    {
        _tabWid->clear();
        _openedTabsNames.clear();
        _openedTabsDirectories.clear();
        _strListModel->setStringList(_openedTabsNames);
        emit  statusBar()->showMessage(tr("Closed all documents."));
        return true;
    }

    SaveDialog* saveDial = new SaveDialog(this);
    saveDial->sltSetSizes(_amountOfChanged, 2);
    int at_tab = 0;
    int at_item = 0;

    for(auto &&name : _openedTabsNames)
    {
        if(qobject_cast<doc_tab_class*>(_tabWid->widget(at_tab))->wasChanged())
        {
            QTableWidgetItem* itemName = new QTableWidgetItem(name);
            QTableWidgetItem* itemDirectory = new QTableWidgetItem(_openedTabsDirectories.at(at_tab));

            qDebug() << itemName << itemDirectory;
            saveDial->sltSetContents(at_item, 0, itemName);
            saveDial->sltSetContents(at_item, 1, itemDirectory);

            at_item++;
        }
        at_tab++;
    }

    auto res = saveDial->exec();

    switch(res)
    {
    case SaveDialog::Accepted:
        sltSaveAll();
        _tabWid->clear();
        _openedTabsNames.clear();
        _openedTabsDirectories.clear();
        _strListModel->setStringList(_openedTabsNames);
        _amountOfChanged = 0;
        emit  statusBar()->showMessage(tr("Closed all documents."));
        return true;

        break;

    case SaveDialog::Rejected:
        _tabWid->clear();
        _openedTabsNames.clear();
        _openedTabsDirectories.clear();
        _strListModel->setStringList(_openedTabsNames);
        _amountOfChanged = 0;
        emit  statusBar()->showMessage(tr("Closed all documents."));
        return true;
        break;
    case -88:
        return false;
        break;
    default:
        return true;
        break;
    }

    return true;

}

void notepad_editor_class::sltTabs() const
{
    _strListModel->setStringList(_openedTabsNames);
    _tableTabsView->setModel(_strListModel);
    _dockTabsWid->setWidget(_tableTabsView);
    _dockTabsWid->show();
}

void notepad_editor_class::sltDelete() const
{
    doc_tab_class* senderPtr = qobject_cast<doc_tab_class*>(_tabWid->currentWidget());
    if(!senderPtr)
        return;

    senderPtr->clear();
}


void notepad_editor_class::sltSelectAll() const
{
    doc_tab_class* senderPtr = qobject_cast<doc_tab_class*>(_tabWid->currentWidget());
    if(!senderPtr)
        return;

    senderPtr->selectAll();
}

void notepad_editor_class::sltFileChanged(const QString& p_tabName)
{
    _amountOfChanged++;
    _tabWid->setTabIcon(_openedTabsNames.indexOf(p_tabName), QIcon(":/pics/Data/Edit.png"));
}

void notepad_editor_class::sltFileBrowser() const
{
    _treeFileSysView->setModel(_sysModel);
    _dockFileSysWid->setWidget(_treeFileSysView);
    _dockFileSysWid->show();
}

void notepad_editor_class::sltOnClose()
{
    sltSaveAll();
}

void notepad_editor_class::sltCopy() const
{
    doc_tab_class* senderPtr = qobject_cast<doc_tab_class*>(_tabWid->currentWidget());
    if(!senderPtr)
        return;

    senderPtr->copy();
}

void notepad_editor_class::sltCut() const
{
    doc_tab_class* senderPtr = qobject_cast<doc_tab_class*>(_tabWid->currentWidget());
    if(!senderPtr)
        return;

    senderPtr->cut();
}


void notepad_editor_class::sltPaste() const
{
    doc_tab_class* senderPtr = qobject_cast<doc_tab_class*>(_tabWid->currentWidget());
    if(!senderPtr)
        return;

    senderPtr->paste();
}